#ifndef __GO_ENGINE_INTERNAL_H__
#define __GO_ENGINE_INTERNAL_H__

#include "cdefs.h"
#include "go_types.h"
#include "go_board.h"
#include "go_engine.h"

#include <stddef.h>
#include <stdint.h>


#define NUM_BOARD_COPY  4


BEGIN_DECLS

struct go_engine {
    int board_size;
    stone_t my_stone_color;
    int move_count;
    float komi;

    int64_t start_time;    /* in usec */
    int64_t max_exec_time; /* in usec */

    coord2d_t prev_pos; /* opponent's move */

    struct go_board gboard_main;
    struct go_board gboard_copy[NUM_BOARD_COPY];

    coord2d_t best_score_array[MAX_BOARD_SIZE * MAX_BOARD_SIZE];
    coord2d_t move_array[MAX_BOARD_SIZE * MAX_BOARD_SIZE];
};


/*
 * Exported functions
 */
int stone_place(struct go_board*, stone_t, coord_t, coord_t);
int string_remove(struct go_board*, stone_t, coord_t, coord_t);

int valid_move_get(struct go_board*, stone_t, coord2d_t*, int, int*);
bool is_valid_move(struct go_board*, stone_t, coord_t, coord_t);


/*
 * Internal functions (but exported to only test functions)
 */
bool is_suicide_move(struct go_board*, stone_t, coord_t, coord_t);
bool is_ko_move(struct go_board*, stone_t, coord_t, coord_t, coord_t*,
                coord_t*);
bool is_surrounded_position(struct go_board*, stone_t, coord_t, coord_t);
bool is_eye_position(struct go_board*, stone_t, coord_t, coord_t);

END_DECLS

#endif /* __GO_ENGINE_INTERNAL_H__ */
