#ifndef __GO_STRING_H__
#define __GO_STRING_H__

#include "cdefs.h"
#include "go_types.h"


#define MAX_BOARD_SIZE          9
#define MAX_STRING_NUM          81 /* XXX */
#define MAX_STRING_SIZE         81 /* XXX */
#define MAX_ADJACENT_NUM        81 /* XXX */

#define STRING_STATUS_INIT      0x00000000
#define STRING_STATUS_ALIVE     0x00000001
#define STRING_STATUS_DEAD      0x00000002
#define STRING_STATUS_NEUTRAL   0x00000004


BEGIN_DECLS

struct go_string {
    string_id_t string_id;
    stone_t color;
    int size;
    int num_liberty;
    int num_adjacent;
    coord2d_t belong_id[MAX_STRING_SIZE];
    string_id_t adjacent_id[MAX_ADJACENT_NUM];
};


struct go_string_list {
    int board_size;
    int num_string;
    bool id_bitmap_array[MAX_STRING_NUM]; // XXX: tentative
    struct go_string string_list[MAX_STRING_NUM];
    string_id_t board_array[MAX_BOARD_SIZE][MAX_BOARD_SIZE];
};


int go_string_list_initialize(struct go_string_list*, int);
int go_string_list_terminate(struct go_string_list*);

int go_string_list_copy_list(struct go_string_list*, struct go_string_list*);


/*
 * New I/Fs
 */
int go_string_list_create_string(struct go_string_list*, stone_t, coord_t,
                                 coord_t, string_id_t*);
int go_string_list_add_stone(struct go_string_list*, stone_t, coord_t, coord_t,
                             string_id_t);
int go_string_list_merge_string(struct go_string_list*, string_id_t,
                                string_id_t);
int go_string_list_destroy_string(struct go_string_list*, string_id_t);

int go_string_list_get_info2(struct go_string_list*, string_id_t, stone_t*,
                             int*, int*);


/*
 * For debugging
 */
int go_string_list_display(struct go_string_list*);


/*
 * Deprecated I/Fs
 */
int go_string_list_clear(struct go_string_list*);
string_id_t go_string_list_get_id(struct go_string_list*, coord_t, coord_t);
int go_string_list_get_string_stone(struct go_string_list*, stone_t, coord_t,
                                    coord_t, int, coord2d_t*, int*);
int go_string_list_get_liberty_number(struct go_string_list*, stone_t, coord_t,
                                      coord_t, int*);
int go_string_list_get_info(struct go_string_list*, string_id_t,
                            struct go_string*);

END_DECLS

#endif /* __GO_STING_H__ */
