#ifndef __UCT_H__
#define __UCT_H__

#include "cdefs.h"


#define UCT_NUM_NODE    32768


BEGIN_DECLS

struct uct_node {
    double value;       /* winrate (color of this node) */
    int num_win;        /* # of wins (my turn) */
    int num_match;      /* # of matches */
    int num_children;   /* # of children */

    int x;
    int y;

    int depth; /* for debugging */

    struct uct_node* parent;
    struct uct_node* next;
    struct uct_node* children;
};

struct uct_tree {
    struct uct_node* root;

    int node_idx;
    struct uct_node node_array[UCT_NUM_NODE];
};


int uct_tree_initialize(struct uct_tree*);
int uct_tree_terminate(struct uct_tree*);

int uct_tree_create_node(struct uct_tree*, struct uct_node**);
int uct_tree_add_child(struct uct_tree*, struct uct_node*, struct uct_node*);
int uct_tree_get_max_child(struct uct_tree*, struct uct_node*,
                           struct uct_node**);
int uct_tree_get_max_child_with_limit(struct uct_tree*, struct uct_node*,
                                      int, struct uct_node**);
int uct_tree_get_top5_children(struct uct_tree*, struct uct_node*,
                               struct uct_node**, int*);
int uct_tree_get_top5_children_with_limit(struct uct_tree*, struct uct_node*,
                                          int, struct uct_node**, int*);
#ifdef notyet
int uct_tree_sort_child(struct uct_tree*, struct uct_node*, struct uct_node**,
                        int, int*);
#endif /* notyet */

/* For debugging */
int uct_tree_dump_tree(struct uct_tree*, struct uct_node*);


END_DECLS

#endif /* __UCT_H__ */
